#Copyright 2015 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from six import moves
from tempest.common.utils import data_utils
import testtools

from tempest.api.image import base
from tempest import config
from tempest import test
from tempest_lib import exceptions as lib_exc

CONF = config.CONF

class BasicAdminOperationsImageTest(base.BaseV2ImageAdminTest):
    """
    Here we test admin operations of images
    """
    @classmethod
    def resource_setup(cls):
        super(BasicAdminOperationsImageTest, cls).resource_setup()
        cls.client = cls.os.image_client_v2
        cls.admin_client = cls.os_adm.image_client_v2

    def test_admin_deactivate_reactivate_image(self):

        # Create Image by non-admin tenant
        image_name = data_utils.rand_name('image')
        body = self.client.create_image(name=image_name,
                                        container_format='bare',
                                        disk_format='raw')
        image_id = body['id']
        # upload an image file
        content = data_utils.random_bytes()
        image_file = moves.cStringIO(content)
        self.client.store_image(image_id, image_file)
        # deactivate image
        self.admin_client.deactivate_image(image_id)
        body = self.client.show_image(image_id)
        self.assertEqual("deactivated", body['status'])
        # reactivate image
        self.admin_client.reactivate_image(image_id)
        body = self.client.show_image(image_id)
        self.assertEqual("active", body['status'])
